import os
import sys
import collections
from App.ModuleService import ModuleService
from importlib import reload

STATUS_OK = 'ok'
STATUS_FAIL = 'fail'
STATUS_ERROR = 'error'

class PythonModuleService(ModuleService):
	def _initInstance(self):
		self.initFileName = '__init__.py'
		self.moduleFileExtension = 'py'
		
	def _initPackage(self):
		sys.path.append(os.path.dirname(self.path))
		self.initModule = __import__(self.packageName)
		if self.initModule:
			if hasattr(self.initModule, 'meta'):
				self.packageMeta = getattr(self.initModule, 'meta')
	
	def _initModuleNames(self):
		modulePath = self.path
		files = os.listdir(modulePath)
		for fname in files:
			if os.path.isfile(os.path.join(modulePath,fname)) and (fname != self.initFileName) and fname[-3:] == '.'+self.moduleFileExtension:
				self.moduleNames.append(fname[:-3])
		self.moduleNames.sort()
	
	def getModule(self, moduleName):
		if not moduleName in self.moduleNames: raise Exception(moduleName + " is not registered in module service")
		fullModuleName = self.packageName + '.' + moduleName
		if fullModuleName not in sys.modules:
			__import__(fullModuleName)
		module = sys.modules[fullModuleName]
		return module

	def reloadModule(self, moduleName):
		return reload(self.getModule(moduleName))

	def getModuleLastResult(self, moduleName):
		result = None
		if moduleName in self.lastResults:
			result = self.lastResults[moduleName]
		return result
		
	def getModuleLastResultStatus(self, moduleName):
		return self.getModuleLastResult(moduleName)['status']
	
	def getModuleLastResultMessage(self, moduleName):
		return self.getModuleLastResult(moduleName)['message']
	
	def getModuleLastResultData(self, moduleName):
		return self.getModuleLastResult(moduleName)['data']
	
	def getModuleMeta(self, moduleName):
		module = self.getModule(moduleName)
		if hasattr(module, 'meta'):
			return getattr(module, 'meta')
		else:
			return {}

	def runModule(self, moduleName, params):
		module = self.reloadModule(moduleName)
		#module = self.getModule(moduleName)
		if hasattr(module, 'run'):
			definition = getattr(module, 'run')
			try:
				result = definition(params, moduleService=self, path=self.path)
			except Exception as ex:
				result = {'status':'error', 'message': repr(ex)}
				
			if isinstance(result, collections.Mapping):
				self.lastResults[moduleName] = result
			else:
				self.lastResults[moduleName] = {
					'status': 'n/a',
					'message': '(invalid result type)',
					'data': result
				}
		else:
			self.lastResults[moduleName] = {
				'status': 'n/a',
				'message': '(nothing to run)',
				'data': None
			}
			
		return self.lastResults[moduleName]
	
	def isModuleHasResultView(self, moduleName):
		return hasattr(self.getModule(moduleName), 'view')
	
	def getModuleResultView(self, moduleName):
		lastResult = self.getModuleLastResult(moduleName)
		if lastResult['status'] == 'ok':
			module = self.getModule(moduleName)
			if hasattr(module, 'view'):
				return getattr(module, 'view')(lastResult, moduleService=self)
	
	def getModuleSource(self, moduleName):
		f = open(os.path.join(self.path, moduleName + '.py'))
		source = f.read()
		f.close()
		return source
	
	def getSession(self):
		return self.session
	
	def getApplicationSession(self):
		from kernel import session
		return session
		
	def destroy(self):
		sys.path.remove(os.path.dirname(self.path))
		del sys.modules[self.packageName]
		for moduleName in self.moduleNames:
			moduleFullName = self.packageName + "." + moduleName
			if moduleFullName in sys.modules: del sys.modules[moduleFullName]
